/*
<H2>Licence Agreement</H2>
<PRE>InterfaceClassFinder package
Copyright (c) 2009 Forward Computing and Control Pty. Ltd.
NSW. Australia,  www.forward.com.au
All rights reserved.

Redistribution of the source of this package, with or without
modification, is NOT permitted.

Redistribution of the compiled java .class files of this package, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions in compiled java .class form must reproduce the above
   copyright notice, this list of conditions and the following disclaimer
   in the documentation and/or other materials provided with the distribution.

2. If the software is modified, the compiled java .class distribution must
   include, in the documentation and/or other materials provided with the
   distribution, a statement that the software has been modified and
   identify the person who was responsable for the modifications.

3. Neither the name of the author nor Forward Computing and Control may be
   used to endorse or promote products derived from this software
   without specific prior written permission.

4.  All advertising materials mentioning features or use of this
    software must display the following acknowledgment:
    "This product includes software developed by Matthew Ford
     and Forward Computing and Control Pty. Ltd.
    (http://www.forward.com.au)."


THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
</PRE>
 */
package au.com.forward.interfaceClassFinder;

import org.objectweb.asm.Attribute;
import org.objectweb.asm.AnnotationVisitor;
import org.objectweb.asm.FieldVisitor;
import org.objectweb.asm.MethodVisitor;
import org.objectweb.asm.Opcodes;

/**
 *
 * @author Matthew
 */
class ClassVisitor implements org.objectweb.asm.ClassVisitor {

    // reused each time
    private ClassInfo classInfo;

    /**
     * Get the classinfo object for this visitor.<br>
     * Can only call this once and must call org.objectweb.asm.ClassReader.accept(classVisitor,0) first.
     * @return
     */
    ClassInfo getClassInfo() {
        if (classInfo == null) {
            throw new IllegalStateException("Need to call org.objectweb.asm.ClassReader.accept(classVisitor,0) first.");
        }
        ClassInfo info = classInfo;
        classInfo = null;
        return info;
    }

    /**
     * Look for the given interface.<br>
     * If not found recursively look at super interfaces.
     * @param version
     * @param access
     * @param name
     * @param signature
     * @param superName
     * @param interfaces
     */
    public void visit(int version, int access, String name,
            String signature, String superName, String[] interfaces) {

        if (interfaces == null) {
            interfaces = new String[0];
        }
        if (superName == null) {
            // for Object only
            superName = "";
        }
        if (name.endsWith("Processor_1")) {
            System.out.println(" found it");
        }
        classInfo = new ClassInfo(name, superName, interfaces, (access&Opcodes.ACC_PUBLIC)>0, (access&Opcodes.ACC_ABSTRACT)>0);
    }

    /**
     * Set the source if interface found
     * @param source
     * @param debug
     */
    public void visitSource(String source, String debug) {
        if (source == null) {
            source = "";
        }
        classInfo.setSource(source + (debug != null ? debug : ""));
    }

    public void visitOuterClass(String owner, String name, String desc) {
    }

    public AnnotationVisitor visitAnnotation(String desc,
            boolean visible) {
        return null;
    }

    public void visitAttribute(Attribute attr) {
    }

    /**
     * Skip inner classes
     * @param name
     * @param outerName
     * @param innerName
     * @param access
     */
    public void visitInnerClass(String name, String outerName,
            String innerName, int access) {
    }

    public FieldVisitor visitField(int access, String name, String desc,
            String signature, Object value) {
        return null;
    }

    /**
     * Look of public no arg constructor.
     * @param access
     * @param name
     * @param desc
     * @param signature
     * @param exceptions
     * @return
     */
    public MethodVisitor visitMethod(int access, String name,
            String desc, String signature, String[] exceptions) {
        if ("<init>".equals(name)) {
            classInfo.setIsInterface(false); // has a constructor
            if (access == Opcodes.ACC_PUBLIC) {
                if ("()V".equals(desc)) {
                    classInfo.setHasNoArgConstructor(true);
                    //"Found default constructor."
                } else {
                   //"Found public constructor with args " + desc);
                }
            } else {
                //"Found non-public constructor with args " + desc);
            }
        }
        return null;
    }

    /**
     */
    public void visitEnd() {
    }

}
